#!/usr/bin/env python
"""
Remove horizontal stripes from ACS WFC post-SM4 data.

Examples
--------

In Python without TEAL:

>>> from acstools import acs_destripe
>>> acs_destripe.clean('uncorrected_flt.fits', 'csck', clobber=False,
...                    maxiter=15, sigrej=2.0)

In Python with TEAL:

>>> from acstools import acs_destripe
>>> from stsci.tools import teal
>>> teal.teal('acs_destripe')

In Pyraf::

    --> import acstools
    --> teal acs_destripe

From command line::

    % ./acs_destripe [-h][-c] input output [maxiter # [sigrej #]]

"""
from __future__ import print_function

# STDLIB
import logging
import os
import sys

# THIRD-PARTY
import numpy as np
from astropy.io import fits

# STSCI
from stsci.tools import parseinput, teal


__taskname__ = 'acs_destripe'
__version__ = '0.4.1'
__vdate__ = '05-Feb-2014'
__author__ = 'Norman Grogin, STScI, March 2012.'


MJD_SM4 = 54967

logging.basicConfig()
LOG = logging.getLogger(__taskname__)
LOG.setLevel(logging.INFO)


class StripeArray(object):
    """Class to handle data array to be destriped."""

    def __init__(self, image):
        self.hdulist = fits.open(image)
        self.ampstring = self.hdulist[0].header['CCDAMP']
        self.flatcorr = self.hdulist[0].header['FLATCORR']
        self.darkcorr = self.hdulist[0].header['DARKCORR']

        self.darktime = self.hdulist[0].header['EXPTIME'] + self.hdulist[0].header['FLASHDUR']
        if self.hdulist[0].header['EXPTIME'] > 0:  # Not BIAS
            self.darktime += 3.0

        self.configure_arrays()

    def configure_arrays(self):
        """Get the SCI and ERR data."""
        self.science = self.hdulist['sci',1].data
        self.err = self.hdulist['err',1].data
        if (self.ampstring == 'ABCD'):
            self.science = np.concatenate(
                (self.science, self.hdulist['sci',2].data[::-1,:]), axis=1)
            self.err = np.concatenate(
                (self.err, self.hdulist['err',2].data[::-1,:]), axis=1)
        self.ingest_dark()
        self.ingest_flatfield()

    def _get_ref_section(self, refaxis1, refaxis2):
        """Get reference file section to use."""

        sizaxis1 = self.hdulist[1].header['SIZAXIS1']
        sizaxis2 = self.hdulist[1].header['SIZAXIS2']
        centera1 = self.hdulist[1].header['CENTERA1']
        centera2 = self.hdulist[1].header['CENTERA2']

        # configure the offset appropriate to left- or right-side of CCD
        if (self.ampstring[0] == 'A' or self.ampstring[0] == 'C'):
            xdelta = 13
        else:
            xdelta = 35

        if sizaxis1 == refaxis1:
            xlo = 0
            xhi = sizaxis1
        else:
            xlo = centera1 - xdelta - sizaxis1 // 2 - 1
            xhi = centera1 - xdelta + sizaxis1 // 2 - 1

        if sizaxis2 == refaxis2:
            ylo = 0
            yhi = sizaxis2
        else:
            ylo = centera2 - sizaxis2 // 2 - 1
            yhi = centera2 + sizaxis2 // 2 - 1

        return xlo, xhi, ylo, yhi

    def ingest_flatfield(self):
        """Process flatfield."""

        flatfile = self.hdulist[0].header['PFLTFILE']

        # if BIAS or DARK, set flatfield to unity
        if flatfile == 'N/A':
            self.invflat = np.ones(self.science.shape)
            return

        hduflat = self.resolve_reffilename(flatfile)

        if (self.ampstring == 'ABCD'):
            self.invflat = np.concatenate(
                (1 / hduflat['sci',1].data,
                 1 / hduflat['sci',2].data[::-1,:]), axis=1)
        else:
            # complex algorithm to determine proper subarray of flatfield to use

            # which amp?
            if (self.ampstring == 'A' or self.ampstring == 'B' or
                    self.ampstring == 'AB'):
                self.invflat = 1 / hduflat['sci',2].data
            else:
                self.invflat = 1 / hduflat['sci',1].data

            # now, which section?
            flataxis1 = hduflat[1].header['NAXIS1']
            flataxis2 = hduflat[1].header['NAXIS2']

            xlo, xhi, ylo, yhi = self._get_ref_section(flataxis1, flataxis2)

            self.invflat = self.invflat[ylo:yhi,xlo:xhi]

        # apply the flatfield if necessary
        if self.flatcorr != 'COMPLETE':
            self.science = self.science * self.invflat
            self.err = self.err * self.invflat

    def ingest_dark(self):
        """Process dark."""

        if self.hdulist[0].header['PCTECORR'] == 'COMPLETE':
            darkfile = self.hdulist[0].header['DRKCFILE']
        else:
            darkfile = self.hdulist[0].header['DARKFILE']

        # if BIAS or DARK, set dark to zeros
        if darkfile == 'N/A':
            self.dark = np.zeros(self.science.shape)
            return

        hdudark = self.resolve_reffilename(darkfile)

        if (self.ampstring == 'ABCD'):
            self.dark = np.concatenate(
                (hdudark['sci',1].data,
                 hdudark['sci',2].data[::-1,:]), axis=1)
        else:
            # complex algorithm to determine proper subarray of dark to use

            # which amp?
            if (self.ampstring == 'A' or self.ampstring == 'B' or
                    self.ampstring == 'AB'):
                self.dark = hdudark['sci',2].data
            else:
                self.dark = hdudark['sci',1].data

            # now, which section?\
            darkaxis1 = hdudark[1].header['NAXIS1']
            darkaxis2 = hdudark[1].header['NAXIS2']

            xlo, xhi, ylo, yhi = self._get_ref_section(darkaxis1, darkaxis2)

            self.dark = self.dark[ylo:yhi,xlo:xhi]

        # Apply the dark subtraction if necessary.
        # Effect of DARK on ERR is insignificant for de-striping.
        if self.darkcorr != 'COMPLETE':
            self.science = self.science - self.dark * self.darktime

    def resolve_reffilename(self, reffile):
        """Resolve the filename into an absolute pathname (if necessary)."""
        refparts = reffile.partition('$')
        if refparts[1] == '$':
            refdir = os.getenv(refparts[0])
            if refdir is None:
                raise ValueError(
                    'Environment variable {0} undefined!'.format(refparts[0]))
            reffile = os.path.join(refdir, refparts[2])
        if not os.path.exists(reffile):
            raise IOError('{0} could not be found!'.format(reffile))

        return(fits.open(reffile))

    def write_corrected(self, output, clobber=False):
        """Write out the destriped data."""

        # un-apply the flatfield if necessary
        if self.flatcorr != 'COMPLETE':
            self.science = self.science / self.invflat
            self.err = self.err / self.invflat

        # un-apply the dark if necessary
        if self.darkcorr != 'COMPLETE':
            self.science = self.science + self.dark * self.darktime

        # reverse the amp merge
        if (self.ampstring == 'ABCD'):
            tmp_1, tmp_2 = np.split(self.science, 2, axis=1)
            self.hdulist['sci',1].data = tmp_1.copy()
            self.hdulist['sci',2].data = tmp_2[::-1,:].copy()

            tmp_1, tmp_2 = np.split(self.err, 2, axis=1)
            self.hdulist['err',1].data = tmp_1.copy()
            self.hdulist['err',2].data = tmp_2[::-1,:].copy()
        else:
            self.hdulist['sci',1].data = self.science.copy()
            self.hdulist['err',1].data = self.err.copy()

        # Write the output
        self.hdulist.writeto(output, clobber=clobber)


def clean(input, suffix, maxiter=15, sigrej=2.0, clobber=False):
    """Remove horizontal stripes from ACS WFC post-SM4 data.

    .. note::

        Input data must be an ACS/WFC FLT image, with 2 SCI extensions.
        Does not work on RAW image.

        Uses the flatfield specified by the image header keyword PFLTFILE.
        If keyword value is 'N/A', as is the case with biases and darks,
        then unity flatfield is used.

        Uses the dark image specified by the image header keyword DARKFILE.
        If keyword value is 'N/A', then dummy dark with zeroes is used.

    Parameters
    ----------
    input : str or list of str
        Input filenames in one of these formats:

            * a Python list of filenames
            * a partial filename with wildcards ('\*flt.fits')
            * filename of an ASN table ('j12345670_asn.fits')
            * an at-file (``@input``)

    suffix : str
        The string to use to add to each input file name to
        indicate an output product. This string will be appended
        to the _flt suffix in each input filename to create the
        new output filename. For example, setting `suffix='csck'`
        will create '\*_flt_csck.fits' images.

    maxiter : int
        This parameter controls the maximum number of iterations
        to perform when computing the statistics used to compute the
        row-by-row corrections.

    sigrej : float
        This parameters sets the sigma level for the rejection applied
        during each iteration of statistics computations for the
        row-by-row corrections.

    clobber : bool
        Specify whether or not to 'clobber' (delete then replace)
        previously generated products with the same names.

    """
    flist, alist = parseinput.parseinput(input)

    for image in flist:
        # Skip processing pre-SM4 images
        if (fits.getval(image, 'EXPSTART') <= MJD_SM4):
            LOG.warn(image + ' is pre-SM4. Skipping...'%image)
            continue

        # Data must be in ELECTRONS
        if (fits.getval(image, 'BUNIT', ext=1) != 'ELECTRONS'):
            LOG.warn(image + ' is not in ELECTRONS. Skipping...')
            continue

        # Skip processing CTECORR-ed images
        if (fits.getval(image, 'PCTECORR') == 'COMPLETE'):
            LOG.warn(image + ' already has PCTECORR applied. Skipping...')
            continue

        # generate output filename for each input based on specification
        # of the output suffix
        output = image.replace('_flt', '_flt_' + suffix)
        LOG.info('Processing ' + image)
        perform_correction(image, output, maxiter=maxiter, sigrej=sigrej,
                           clobber=clobber)
        LOG.info(output + ' created')


def perform_correction(image, output, maxiter=15, sigrej=2.0, clobber=False):
    """
    Called by `clean` for each input image.

    Parameters
    ----------
    image : str
        Input image name.

    output : str
        Output image name.

    maxiter, sigrej, clobber : see `clean`

    """

    # construct the frame to be cleaned, including the
    # associated data stuctures needed for cleaning
    frame = StripeArray(image)

    # Do the stripe cleaning
    clean_streak(frame, maxiter=maxiter, sigrej=sigrej)

    frame.write_corrected(output, clobber=clobber)


def clean_streak(image, maxiter=15, sigrej=2.0):
    """
    Apply destriping algorithm to input array.

    Parameters
    ----------
    image : `StripeArray` object
        Arrays are modifed in-place.

    maxiter, sigrej : see `clean`

    """

    # create the array to hold the stripe amplitudes
    corr = np.empty(image.science.shape[0])

    # loop over rows to fit the stripe amplitude
    for i in range(image.science.shape[0]):
        # row-by-row iterative sigma-clipped mean; sigma, iters are adjustable
        SMean, SSig, SMedian, SMask = djs_iterstat(
            image.science[i], MaxIter=maxiter, SigRej=sigrej)

        # SExtractor-esque central value statistic; slightly sturdier against
        # skewness of pixel histogram due to faint source flux
        corr[i] = 2.5 * SMedian - 1.5 * SMean

    # preserve the original mean level of the image
    corr -= np.average(corr)

    # apply the correction row-by-row
    for i in range(image.science.shape[0]):
        # stripe is constant along the row, before flatfielding;
        # afterwards it has the shape of the inverse flatfield
        truecorr = corr[i] * image.invflat[i] / np.average(image.invflat[i])

        # correct the SCI extension
        image.science[i] -= truecorr

        # correct the ERR extension
        image.err[i] = np.sqrt(np.abs(image.err[i]**2 - truecorr))


def djs_iterstat(InputArr, MaxIter=10, SigRej=3.0, Max=None, Min=None,
                 RejVal=None):
    """
    Iterative sigma-clipping.

    Parameters
    ----------
    InputArr : `numpy.ndarray`
        Input image array.

    MaxIter, SigRej : see `clean`

    Max, Min : float
        Max and min values for clipping.

    RejVal : float
        Array value to reject, in addition to clipping.

    Returns
    -------
    FMean, FSig, FMedian : float
        Mean, sigma, and median of final result.

    SaveMask : `numpy.ndarray`
        Image mask from the final iteration.

    """
    NGood    = InputArr.size
    ArrShape = InputArr.shape
    if NGood == 0:
        LOG.warn('djs_iterstat - No data points given')
        return 0, 0, 0, 0
    if NGood == 1:
        LOG.warn('djs_iterstat - Only one data point; cannot compute stats')
        return 0, 0, 0, 0
    if np.unique(InputArr).size == 1:
        LOG.warn('djs_iterstat - Only one value in data; cannot compute stats')
        return 0, 0, 0, 0

    # Determine Max and Min
    if Max is None:
        Max = InputArr.max()
    if Min is None:
        Min = InputArr.min()

    Mask = np.zeros(ArrShape, dtype=np.byte) + 1

    # Reject those above Max and those below Min
    Mask[InputArr > Max] = 0
    Mask[InputArr < Min] = 0
    if RejVal is not None:
        Mask[InputArr == RejVal] = 0
    FMean = np.sum(1.0 * InputArr * Mask) / NGood
    FSig  = np.sqrt(np.sum((1.0 * InputArr - FMean)**2 * Mask) / (NGood - 1))

    NLast = -1
    Iter  =  0
    NGood = np.sum(Mask)
    if NGood < 2:
        LOG.warn('djs_iterstat - No good data points; cannot compute stats')
        return -1, -1, -1, -1

    while (Iter < MaxIter) and (NLast != NGood) and (NGood >= 2):
        LoVal = FMean - SigRej * FSig
        HiVal = FMean + SigRej * FSig
        NLast = NGood

        Mask[InputArr < LoVal] = 0
        Mask[InputArr > HiVal] = 0
        NGood = np.sum(Mask)

        if NGood >= 2:
            FMean = np.sum(1.0 * InputArr * Mask) / NGood
            FSig  = np.sqrt(np.sum(
                (1.0 * InputArr - FMean)**2 * Mask) / (NGood - 1))

        SaveMask = Mask.copy()

        Iter += 1

    if np.sum(SaveMask) > 2:
        FMedian = np.median(InputArr[SaveMask == 1])
    else:
        FMedian = FMean

    return FMean, FSig, FMedian, SaveMask


#-------------------------#
# Interfaces used by TEAL #
#-------------------------#


def getHelpAsString(fulldoc=True):
    """
    Returns documentation on the `clean` function. Required by TEAL.

    """
    return clean.__doc__


def run(configobj=None):
    """
    TEAL interface for the `clean` function.

    """
    clean(configobj['input'],
          configobj['suffix'],
          maxiter=configobj['maxiter'],
          sigrej=configobj['sigrej'],
          clobber=configobj['clobber'])


#-----------------------------#
# Interfaces for command line #
#-----------------------------#


def main():
    """Command line driver."""
    import getopt

    usg_str = 'USAGE: acs_destripe [-h][-c] input output [maxiter # [sigrej #]]'

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'hc')
    except getopt.error as e:
      print(str(e))
      print(usg_str)
      print('\t', __version__)

    # initialize default values
    help = False
    clobber = False
    maxiter = 15
    sigrej = 2.0

    # read options
    for opt, value in optlist:
        if opt == '-h':
            help = True
        elif opt == '-c':
            clobber = True

    if help:
        print(usg_str)
        print('\t', __version__, '(', __vdate__, ')')
        return

    if len(args) < 2:
        LOG.error(usg_str)
        return

    if len(args) > 2:
        # User provided parameters for maxiter, and possibly sigrej
        maxiter = int(args[2])
        if len(args) == 4:
            sigrej = float(args[3])

    clean(args[0], args[1], clobber=clobber, maxiter=maxiter, sigrej=sigrej)


if __name__ == '__main__':
    main()



"""
The calacs module contains a function `calacs` that calls the CALACS executable.
Use this function to facilitate batch runs of CALACS, or for the TEAL interface.

Examples
--------

In Python without TEAL:

>>> from acstools import calacs
>>> calacs.calacs(filename)

In Python with TEAL:

>>> from stsci.tools import teal
>>> from acstools import calacs
>>> teal.teal('calacs')

In Pyraf::

    --> import acstools
    --> epar calacs

"""

import os.path
import subprocess


def calacs(input_file, exec_path=None, time_stamps=False, temp_files=False,
           verbose=False, debug=False, quiet=False, single_core=False):
    """
    Run the calacs.e executable as from the shell.

    By default this will run the calacs given by 'calacs.e'.

    Parameters
    ----------
    input_file : str
        Name of input file.

    exec_path : str, optional
        The complete path to a calacs executable.

    time_stamps : bool, optional
        Set to True to turn on the printing of time stamps.

    temp_files : bool, optional
        Set to True to have CALACS save temporary files.

    verbose : bool, optional
        Set to True for verbose output.

    debug : bool, optional
        Set to True to turn on debugging output.

    quiet : bool, optional
        Set to True for quiet output.

    single_core : bool, optional
        CTE correction in CALACS will by default try to use all available
        CPUs on your computer. Set this to True to force the use of just
        one CPU.

    """
    if exec_path:
        if not os.path.exists(exec_path):
            raise OSError('Executable not found: ' + exec_path)

        call_list = [exec_path]
    else:
        call_list = ['calacs.e']

    if time_stamps:
        call_list.append('-t')

    if temp_files:
        call_list.append('-s')

    if verbose:
        call_list.append('-v')

    if debug:
        call_list.append('-d')

    if quiet:
        call_list.append('-q')

    if single_core:
        call_list.append('-1')

    if not os.path.exists(input_file):
        raise IOError('Input file not found: ' + input_file)

    call_list.append(input_file)

    subprocess.call(call_list)


def getHelpAsString():
    """
    Returns documentation on the `calacs` function. Required by TEAL.

    """
    return calacs.__doc__


def run(configobj=None):
    """
    TEAL interface for the `calacs` function.

    """
    calacs(configobj['input_file'],
           exec_path=configobj['exec_path'],
           time_stamps=configobj['time_stamps'],
           temp_files=configobj['temp_files'],
           verbose=configobj['verbose'],
           debug=configobj['debug'],
           quiet=configobj['quiet'],
           single_core=configobj['single_core'])



"""
The acs2d module contains a function `acs2d` that calls the ACS2D executable.
Use this function to facilitate batch runs of ACS2D, or for the TEAL interface.

Examples
--------

In Python without TEAL:

>>> from acstools import acs2d
>>> acs2d.acs2d('*blv_tmp.fits')

In Python with TEAL:

>>> from stsci.tools import teal
>>> from acstools import acs2d
>>> teal.teal('acs2d')

In Pyraf::

    --> import acstools
    --> epar acs2d

"""
# STDLIB
import os.path
import subprocess

# STSCI
from stsci.tools import parseinput
try:
    from stsci.tools import teal
except:
    teal = None


__taskname__ = "acs2d"
__version__ = "1.0"
__vdate__ = "18-Dec-2012"


def acs2d(input, exec_path='', time_stamps=False, verbose=False, quiet=False,
          dqicorr=False, glincorr=False, lflgcorr=False, darkcorr=False,
          flshcorr=False, flatcorr=False, shadcorr=False, photcorr=False):
    """
    Run the acs2d.e executable as from the shell.

    Output is automatically named based on input suffix:

        +--------------------+----------------+------------------------------+
        | INPUT              | OUTPUT         | EXPECTED DATA                |
        +====================+================+==============================+
        | ``*_raw.fits``     | ``*_flt.fits`` | SBC image.                   |
        +--------------------+----------------+------------------------------+
        | ``*_blv_tmp.fits`` | ``*_flt.fits`` | ACSCCD output.               |
        +--------------------+----------------+------------------------------+
        | ``*_blc_tmp.fits`` | ``*_flc.fits`` | ACSCCD output with PCTECORR. |
        +--------------------+----------------+------------------------------+
        | ``*_crj_tmp.fits`` | ``*_crj.fits`` | ACSREJ output.               |
        +--------------------+----------------+------------------------------+
        | ``*_crc_tmp.fits`` | ``*_crc.fits`` | ACSREJ output with PCTECORR. |
        +--------------------+----------------+------------------------------+

    Parameters
    ----------
    input : str or list of str
        Input filenames in one of these formats:

            * a single filename ('j1234567q_blv_tmp.fits')
            * a Python list of filenames
            * a partial filename with wildcards ('\*blv_tmp.fits')
            * filename of an ASN table ('j12345670_asn.fits')
            * an at-file (``@input``)

    exec_path : str, optional
        The complete path to ACS2D executable.
        If not given, run ACS2D given by 'acs2d.e'.

    time_stamps : bool, optional
        Set to True to turn on the printing of time stamps.

    verbose : bool, optional
        Set to True for verbose output.

    quiet : bool, optional
        Set to True for quiet output.

    dqicorr, glincorr, lflgcorr, darkcorr, flshcorr, flatcorr, shadcorr, photcorr : bool, optional
        Enable XXXXCORR.
        If all False, will set all but FLSHCORR and SHADCORR to PERFORM.
        If any is True, will set that to PERFORM and the rest to OMIT.
        GLINCORR and LFLGCORR are used for SBC MAMA only.
        FLSHCORR and SHADCORR are used for CCD only.

    """
    if exec_path:
        if not os.path.exists(exec_path):
            raise OSError('Executable not found: ' + exec_path)
        call_list = [exec_path]
    else:
        call_list = ['acs2d.e']

    # Parse input to get list of filenames to process.
    # acs2d.e only takes 'file1,file2,...'
    infiles, dummy_out = parseinput.parseinput(input)
    call_list.append(','.join(infiles))

    if time_stamps:
        call_list.append('-t')

    if verbose:
        call_list.append('-v')

    if quiet:
        call_list.append('-q')

    if dqicorr:
        call_list.append('-dqi')

    if glincorr:
        call_list.append('-glin')

    if lflgcorr:
        call_list.append('-lflg')

    if darkcorr:
        call_list.append('-dark')

    if flshcorr:
        call_list.append('-flash')

    if flatcorr:
        call_list.append('-flat')

    if shadcorr:
        call_list.append('-shad')

    if photcorr:
        call_list.append('-phot')

    subprocess.call(call_list)


def getHelpAsString():
    """
    Returns documentation on the `acs2d` function. Required by TEAL.

    """
    return acs2d.__doc__


def run(configobj=None):
    """
    TEAL interface for the `acs2d` function.

    """
    acs2d(configobj['input'],
          exec_path=configobj['exec_path'],
          time_stamps=configobj['time_stamps'],
          verbose=configobj['verbose'],
          quiet=configobj['quiet'],
          dqicorr=configobj['dqicorr'],
          glincorr=configobj['glincorr'],
          lflgcorr=configobj['lflgcorr'],
          darkcorr=configobj['darkcorr'],
          flshcorr=configobj['flshcorr'],
          flatcorr=configobj['flatcorr'],
          shadcorr=configobj['shadcorr'],
          photcorr=configobj['photcorr'])



from __future__ import division # confidence high

import sys
import distutils
import distutils.core
import distutils.sysconfig

try:
    import numpy
except:
    raise ImportError('NUMPY was not found. It may not be installed or '
                      'it may not be on your PYTHONPATH')


pythoninc = distutils.sysconfig.get_python_inc()
numpyinc = numpy.get_include()

pkg = 'acstools'

ext = [ distutils.core.Extension(
    pkg + '.PixCte_FixY',
    ['src/PixCte_FixY.c', 'src/PixCteCorr_funcs.c', 'src/FixYCte.c'],
    include_dirs = [pythoninc,numpyinc] ) ]

setupargs = {
    'version': '1.7.1',
    'description': 'Python Tools for ACS Data',
    'author': 'Warren Hack, Matt Davis, Pey Lian Lim, '
              'Jay Anderson, Norman Grogin',
    'author_email': 'help@stsci.edu',
    'license': 'http://www.stsci.edu/resources/software_hardware/pyraf/LICENSE',
    'data_files': [(pkg + '/pars', ['lib/acstools/pars/*']),
                   (pkg, ['LICENSE.txt'])],
    'scripts': ['lib/acstools/acs_destripe', 'lib/acstools/runastrodriz'],
    'platforms': ['Linux', 'Solaris', 'Mac OS X', 'Win'],
    'ext_modules': ext,
    'package_dir': {'acstools': 'lib/acstools'}
}



from nose import tools

from acstools import calacs

@tools.raises(OSError)
def test_raises_oserror():
    exec_path = 'IamNOTanEXECUTABLE'

    input_file = 'IamNOTaFILE'

    calacs.calacs(input_file, exec_path=exec_path)


@tools.raises(IOError)
def test_raises_ioerror():
    input_file = 'IamNOTaFILE'

    calacs.calacs(input_file)



"""The acstools package holds Python tasks useful for analyzing ACS data.

These tasks include:

Utility and library functions used by these tasks are also included in this
module.


"""

from .version import *

import acs_destripe
import PixCteCorr
import runastrodriz
import calacs
import acsccd
import acscte
import acs2d
import acsrej
import acssum

# These lines allow TEAL to print out the names of TEAL-enabled tasks
# upon importing this package.
import os
from stsci.tools import teal
teal.print_tasknames(__name__, os.path.dirname(__file__))



"""
The acsccd module contains a function `acsccd` that calls the ACSCCD executable.
Use this function to facilitate batch runs of ACSCCD, or for the TEAL interface.

.. warning:: Do not use with SBC MAMA images.

Examples
--------

In Python without TEAL:

>>> from acstools import acsccd
>>> acsccd.acsccd('*raw.fits')

In Python with TEAL:

>>> from stsci.tools import teal
>>> from acstools import acsccd
>>> teal.teal('acsccd')

In Pyraf::

    --> import acstools
    --> epar acsccd

"""
# STDLIB
import os.path
import subprocess

# STSCI
from stsci.tools import parseinput
try:
    from stsci.tools import teal
except:
    teal = None


__taskname__ = "acsccd"
__version__ = "2.0"
__vdate__ = "13-Aug-2013"


#
# These keywords do not work because they get overwritten by header in
# acsccd/getacsflags.c
#
# dqicorr=False, atodcorr=False, blevcorr=False, biascorr=False
#
# dqicorr, atodcorr, blevcorr, biascorr : bool, optional
#     Enable XXXXCORR.
#     If all False, will set all but ATODCORR to PERFORM.
#     If any is True, will set that to PERFORM and the rest to OMIT.
#
def acsccd(input, exec_path='', time_stamps=False, verbose=False, quiet=False):
    """
    Run the acsccd.e executable as from the shell.

    Expect input to be ``*_raw.fits``.
    Output is automatically named ``*_blv_tmp.fits``.

    .. note:: Calibration flags are controlled by primary header.

    .. warning:: Do not use with SBC MAMA images.

    Parameters
    ----------
    input : str or list of str
        Input filenames in one of these formats:

            * a single filename ('j1234567q_raw.fits')
            * a Python list of filenames
            * a partial filename with wildcards ('\*raw.fits')
            * filename of an ASN table ('j12345670_asn.fits')
            * an at-file (``@input``)

    exec_path : str, optional
        The complete path to ACSCCD executable.
        If not given, run ACSCCD given by 'acsccd.e'.

    time_stamps : bool, optional
        Set to True to turn on the printing of time stamps.

    verbose : bool, optional
        Set to True for verbose output.

    quiet : bool, optional
        Set to True for quiet output.

    """
    if exec_path:
        if not os.path.exists(exec_path):
            raise OSError('Executable not found: ' + exec_path)
        call_list = [exec_path]
    else:
        call_list = ['acsccd.e']

    # Parse input to get list of filenames to process.
    # acsccd.e only takes 'file1,file2,...'
    infiles, dummy_out = parseinput.parseinput(input)
    call_list.append(','.join(infiles))

    if time_stamps:
        call_list.append('-t')

    if verbose:
        call_list.append('-v')

    if quiet:
        call_list.append('-q')

    #if dqicorr:
    #    call_list.append('-dqi')

    #if atodcor:
    #    call_list.append('-atod')

    #if blevcorr:
    #    call_list.append('-blev')

    #if biascorr:
    #    call_list.append('-bias')

    subprocess.call(call_list)


def getHelpAsString():
    """
    Returns documentation on the `acsccd` function. Required by TEAL.

    """
    return acsccd.__doc__


def run(configobj=None):
    """
    TEAL interface for the `acsccd` function.

    """
    acsccd(configobj['input'],
           exec_path=configobj['exec_path'],
           time_stamps=configobj['time_stamps'],
           verbose=configobj['verbose'],
           quiet=configobj['quiet'] #,
           #dqicorr=configobj['dqicorr'],
           #atodcorr=configobj['atodcorr'],
           #blevcorr=configobj['blevcorr'],
           #biascorr=configobj['biascorr']
           )


# Taken out from pars/acsccd.cfg
#
#dqicorr = False
#atodcorr = False
#blevcorr = False
#biascorr = False

# Taken out from pars/acsccd.cfgspc
#
#dqicorr = boolean_kw(default=False, comment="Initialize DQ array")
#atodcorr = boolean_kw(default=False, comment="Apply A-to-D correction")
#blevcorr = boolean_kw(default=False, comment="Subtract bias from overscan")
#biascorr = boolean_kw(default=False, comment="Subtract bias image")



"""
The acscte module contains a function `acscte` that calls the ACSCTE executable.
Use this function to facilitate batch runs of ACSCTE, or for the TEAL interface.

.. warning:: Only use with WFC full-frame images.

Examples
--------

In Python without TEAL:

>>> from acstools import acscte
>>> acscte.acscte('*blv_tmp.fits')

In Python with TEAL:

>>> from stsci.tools import teal
>>> from acstools import acscte
>>> teal.teal('acscte')

In Pyraf::

    --> import acstools
    --> epar acscte

"""
# STDLIB
import os.path
import subprocess

# STSCI
from stsci.tools import parseinput
try:
    from stsci.tools import teal
except:
    teal = None


__taskname__ = "acscte"
__version__ = "1.0"
__vdate__ = "13-Aug-2013"


def acscte(input, exec_path='', time_stamps=False, verbose=False, quiet=False,
           single_core=False):
    """
    Run the acscte.e executable as from the shell.

    Expect input to be ``*_blv_tmp.fits``.
    Output is automatically named ``*_blc_tmp.fits``.

    .. note:: Calibration flags are controlled by primary header.

    .. warning:: Only use with WFC full-frame images.

    Parameters
    ----------
    input : str or list of str
        Input filenames in one of these formats:

            * a single filename ('j1234567q_blv_tmp.fits')
            * a Python list of filenames
            * a partial filename with wildcards ('\*blv_tmp.fits')
            * filename of an ASN table ('j12345670_asn.fits')
            * an at-file (``@input``)

    exec_path : str, optional
        The complete path to ACSCTE executable.
        If not given, run ACSCTE given by 'acscte.e'.

    time_stamps : bool, optional
        Set to True to turn on the printing of time stamps.

    verbose : bool, optional
        Set to True for verbose output.

    quiet : bool, optional
        Set to True for quiet output.

    single_core : bool, optional
        CTE correction in ACSCTE will by default try to use all available
        CPUs on your computer. Set this to True to force the use of just
        one CPU.

    """
    if exec_path:
        if not os.path.exists(exec_path):
            raise OSError('Executable not found: ' + exec_path)
        call_list = [exec_path]
    else:
        call_list = ['acscte.e']

    # Parse input to get list of filenames to process.
    # acscte.e only takes 'file1,file2,...'
    infiles, dummy_out = parseinput.parseinput(input)
    call_list.append(','.join(infiles))

    if time_stamps:
        call_list.append('-t')

    if verbose:
        call_list.append('-v')

    if quiet:
        call_list.append('-q')

    if single_core:
        call_list.append('-1')

    subprocess.call(call_list)


def getHelpAsString():
    """
    Returns documentation on the `acscte` function. Required by TEAL.

    """
    return acscte.__doc__


def run(configobj=None):
    """
    TEAL interface for the `acscte` function.

    """
    acscte(configobj['input'],
           exec_path=configobj['exec_path'],
           time_stamps=configobj['time_stamps'],
           verbose=configobj['verbose'],
           quiet=configobj['quiet'],
           single_core=configobj['single_core']
           )



#!/usr/bin/env python

""" runastrodriz.py - Module to control operation of astrodrizzle to
        remove distortion and combine HST images in the pipeline.

USAGE: runastrodriz.py [-fhibn] inputFilename [newpath]

Alternative USAGE:
    python
    from acstools import runastrodriz
    runastrodriz.process(inputFilename,force=False,newpath=None,inmemory=False)

GUI Usage under Python:
    python
    from stsci.tools import teal
    import acstools
    cfg = teal.teal('runastrodriz')

PyRAF Usage:
    epar runastrodriz

If the '-i' option gets specified, no intermediate products will be written out
to disk. These products, instead, will be kept in memory. This includes all
single drizzle products (*single_sci and *single_wht), median image,
blot images, and crmask images.  The use of this option will therefore require
significantly more memory than usual to process the data.

If a value has been provided for the newpath parameter, all processing will be
performed in that directory/ramdisk.  The steps involved are:
   - create a temporary directory under that directory named after the input file
   - copy all files related to the input to that new directory
   - change to that new directory and run astrodrizzle
   - change back to original directory
   - move (not copy) ALL files from temp directory to original directory
   - delete temp sub-directory

The '-b' option will run this task in BASIC mode without creating headerlets
for each input image.

The '-n' option allows the user to specify the number of cores to be used in
running AstroDrizzle.

*** INITIAL VERSION
W.J. Hack  12 Aug 2011: Initial version based on Version 1.2.0 of
                        STSDAS$pkg/hst_calib/wfc3/runwf3driz.py
W.J. Hack  27 Jun 2012: Implement support to process in different directory

W.J. Hack  24 Aug 2012: Provided interface for in-memory option

W.J. Hack  26 Nov 2012: Option to write out headerlets added and debugged

"""
# Import standard Python modules
from __future__ import division # confidence high
import glob
import os
import shutil
import string
import sys
import time

# THIRD-PARTY
from astropy.io import fits
from stsci.tools import fileutil, asnutil


__taskname__ = "runastrodriz"

# Local variables
__version__ = "1.5.2"
__vdate__ = "(03-Apr-2013)"

# Define parameters which need to be set specifically for
#    pipeline use of astrodrizzle
pipeline_pars = {'mdriztab':True,
                 'stepsize':10,
                 'output':'',
                 'updatewcs':True,
                 'preserve':False,
                 'resetbits':4096}

#default marker for trailer files
__trlmarker__ = '*** astrodrizzle Processing Version '+__version__+__vdate__+'***\n'

# History:
# Version 1.0.0 - Derived from v1.2.0 of wfc3.runwf3driz to run astrodrizzle


#### TEAL Interfaces
def getHelpAsString():
    helpString = 'runastrodriz Version '+__version__+__vdate__+'\n'
    helpString += __doc__+'\n'

    return helpString


def help():
    print getHelpAsString()


def run(configobj=None):
    process(configobj['input'],force=configobj['force'],
                newpath=configobj['newpath'],inmemory=configobj['in_memory'],
                num_cores=configobj['num_cores'],headerlets=configobj['headerlets'])


#### Primary user interface
def process(inFile,force=False,newpath=None, inmemory=False, num_cores=None,
            headerlets=True):
    """ Run astrodrizzle on input file/ASN table
        using default values for astrodrizzle parameters.
    """
    # We only need to import this package if a user run the task
    import drizzlepac
    from drizzlepac import processInput # used for creating new ASNs for _flc inputs
    import stwcs

    if headerlets:
        from stwcs.wcsutil import headerlet

    # Open the input file
    try:
        # Make sure given filename is complete and exists...
        inFilename = fileutil.buildRootname(inFile,ext=['.fits'])
        if not os.path.exists(inFilename):
            print "ERROR: Input file - %s - does not exist." % inFilename
            return
    except TypeError:
        print "ERROR: Inappropriate input file."
        return

    #If newpath was specified, move all files to that directory for processing
    if newpath:
        orig_processing_dir = os.getcwd()
        new_processing_dir = _createWorkingDir(newpath,inFilename)
        _copyToNewWorkingDir(new_processing_dir,inFilename)
        os.chdir(new_processing_dir)

    # Initialize for later use...
    _mname = None
    _new_asn = None
    _calfiles = []

    # Check input file to see if [DRIZ/DITH]CORR is set to PERFORM
    if '_asn' in inFilename:
        # We are working with an ASN table.
        # Use asnutil code to extract filename
        inFilename = _lowerAsn(inFilename)
        _new_asn = [inFilename]
        _asndict = asnutil.readASNTable(inFilename,None,prodonly=False)
        _fname = fileutil.buildRootname(string.lower(_asndict['output']),ext=['_drz.fits'])
        _cal_prodname = string.lower(_asndict['output'])

        # Retrieve the first member's rootname for possible use later
        _fimg = fits.open(inFilename)
        for name in _fimg[1].data.field('MEMNAME'):
            if name[-1] != '*':
                _mname = string.lower(string.split(name,'\0',1)[0])
                break
        _fimg.close()
        del _fimg

    else:
        # Check to see if input is a _RAW file
        # If it is, strip off the _raw.fits extension...
        _indx = string.find(inFilename,'_raw')
        if _indx < 0: _indx = len(inFilename)
        # ... and build the CALXXX product rootname.
        _mname = fileutil.buildRootname(inFilename[:_indx])
        _cal_prodname = inFilename[:_indx]
        # Reset inFilename to correspond to appropriate input for
        # drizzle: calibrated product name.
        inFilename = _mname

        if _mname == None:
            errorMsg = 'Could not find calibrated product!'
            raise Exception,errorMsg

    # Create trailer filenames based on ASN output filename or
    # on input name for single exposures
    if '_raw' in inFile:
        # Output trailer file to RAW file's trailer
        _trlroot = inFile[:inFile.find('_raw')]
    elif '_asn' in inFile:
        # Output trailer file to ASN file's trailer, not product's trailer
        _trlroot = inFile[:inFile.find('_asn')]
    else:
        # Default: trim off last suffix of input filename
        # and replacing with .tra
        _indx = inFile.rfind('_')
        if _indx > 0:
            _trlroot = inFile[:_indx]
        else:
            _trlroot = inFile

    _trlfile = _trlroot + '.tra'

    # Open product and read keyword value
    # Check to see if product already exists...
    dkey = 'DRIZCORR'
    # ...if product does NOT exist, interrogate input file
    # to find out whether 'dcorr' has been set to PERFORM
    # Check if user wants to process again regardless of DRIZCORR keyword value
    if force: dcorr = 'PERFORM'
    else:
        if _mname :
            _fimg = fits.open(fileutil.buildRootname(_mname,ext=['_raw.fits']))
            _phdr = _fimg['PRIMARY'].header
            if dkey in _phdr:
                dcorr = _phdr[dkey]
            else:
                dcorr = None
            _fimg.close()
            del _fimg
        else:
            dcorr = None

    time_str = _getTime()
    _tmptrl = _trlroot + '_tmp.tra'
    _drizfile = _trlroot + '_pydriz'
    _drizlog = _drizfile + ".log" # the '.log' gets added automatically by astrodrizzle
    if dcorr == 'PERFORM':
        if '_asn.fits' not in inFilename:
            # Working with a singleton
            # However, we always want to make sure we always use
            # a calibrated product as input, if available.
            _infile = fileutil.buildRootname(_cal_prodname)
            _infile_flc = fileutil.buildRootname(_cal_prodname,ext=['_flc.fits'])

            _cal_prodname = _infile
            _inlist = _calfiles = [_infile]

            # Add CTE corrected filename as additional input if present
            if os.path.exists(_infile_flc) and _infile_flc != _infile:
                _inlist.append(_infile_flc)

        else:
            # Working with an ASN table...
            _infile = inFilename
            flist,duplist = processInput.checkForDuplicateInputs(_asndict['order'])
            _calfiles = flist
            if len(duplist) > 0:
                origasn = processInput.changeSuffixinASN(inFilename,'flt')
                dupasn = processInput.changeSuffixinASN(inFilename,'flc')
                _inlist = [origasn,dupasn]
            else:
                _inlist = [_infile]
            # We want to keep the original specification of the calibration
            # product name, though, not a lower-case version...
            _cal_prodname = inFilename
            _new_asn.extend(_inlist) # kept so we can delete it when finished


        # Run astrodrizzle and send its processing statements to _trlfile
        _pyver = drizzlepac.astrodrizzle.__version__

        for _infile in _inlist: # Run astrodrizzle for all inputs
            # Create trailer marker message for start of astrodrizzle processing
            _trlmsg = _timestamp('astrodrizzle started ')
            _trlmsg = _trlmsg+ __trlmarker__
            _trlmsg = _trlmsg + '%s: Processing %s with astrodrizzle Version %s\n' % (time_str,_infile,_pyver)
            print _trlmsg

            # Write out trailer comments to trailer file...
            ftmp = open(_tmptrl,'w')
            ftmp.writelines(_trlmsg)
            ftmp.close()
            _appendTrlFile(_trlfile,_tmptrl)

            _pyd_err = _trlroot+'_pydriz.stderr'

            try:
                b = drizzlepac.astrodrizzle.AstroDrizzle(input=_infile,runfile=_drizfile,
                                            configobj='defaults',in_memory=inmemory,
                                            num_cores=num_cores, **pipeline_pars)
            except Exception, errorobj:
                _appendTrlFile(_trlfile,_drizlog)
                _appendTrlFile(_trlfile,_pyd_err)
                _ftrl = open(_trlfile,'a')
                _ftrl.write('ERROR: Could not complete astrodrizzle processing of %s.\n' % _infile)
                _ftrl.write(str(sys.exc_type)+': ')
                _ftrl.writelines(str(errorobj))
                _ftrl.write('\n')
                _ftrl.close()
                print 'ERROR: Could not complete astrodrizzle processing of %s.' % _infile
                raise Exception, str(errorobj)

            # Now, append comments created by PyDrizzle to CALXXX trailer file
            print 'Updating trailer file %s with astrodrizzle comments.' % _trlfile
            _appendTrlFile(_trlfile,_drizlog)

        # Save this for when astropy.io.fits can modify a file 'in-place'
        # Update calibration switch
        _fimg = fits.open(_cal_prodname, mode='update')
        _fimg['PRIMARY'].header[dkey] = 'COMPLETE'
        _fimg.close()
        del _fimg

        # Enforce pipeline convention of all lower-case product
        # names
        _prodlist = glob.glob('*drz.fits')
        for _prodname in _prodlist:
            _plower = _prodname.lower()
            if _prodname != _plower:  os.rename(_prodname,_plower)

    else:
        # Create default trailer file messages when astrodrizzle is not
        # run on a file.  This will typically apply only to BIAS,DARK
        # and other reference images.
        # Start by building up the message...
        _trlmsg = _timestamp('astrodrizzle skipped ')
        _trlmsg = _trlmsg + __trlmarker__
        _trlmsg = _trlmsg + '%s: astrodrizzle processing not requested for %s.\n' % (time_str,inFilename)
        _trlmsg = _trlmsg + '       astrodrizzle will not be run at this time.\n'
        print _trlmsg

        # Write message out to temp file and append it to full trailer file
        ftmp = open(_tmptrl,'w')
        ftmp.writelines(_trlmsg)
        ftmp.close()
        _appendTrlFile(_trlfile,_tmptrl)

    _fmsg = None
    # Append final timestamp to trailer file...
    _final_msg = '%s: Finished processing %s \n' % (time_str,inFilename)
    _final_msg += _timestamp('astrodrizzle completed ')
    _trlmsg += _final_msg
    ftmp = open(_tmptrl,'w')
    ftmp.writelines(_trlmsg)
    ftmp.close()
    _appendTrlFile(_trlfile,_tmptrl)

    # If we created a new ASN table, we need to remove it
    if _new_asn != None:
        for _name in _new_asn: fileutil.removeFile(_name)

    # Clean up any generated OrIg_files directory
    if os.path.exists("OrIg_files"):
        # check to see whether this directory is empty
        flist = glob.glob('OrIg_files/*.fits')
        if len(flist) == 0:
            os.rmdir("OrIg_files")
        else:
            print 'OrIg_files directory NOT removed as it still contained images...'
    if headerlets:
        # Generate headerlets for each updated FLT image
        hlet_msg = _timestamp("Writing Headerlets started")
        for fname in _calfiles:
            frootname = fileutil.buildNewRootname(fname)
            hname = "%s_flt_hlet.fits"%frootname
            hlet_msg += "Created Headerlet file %s \n"%hname
            try:
                headerlet.write_headerlet(fname,'OPUS',output='flt', wcskey='PRIMARY',
                    author="OPUS",descrip="Default WCS from Pipeline Calibration",
                    attach=False,clobber=True,logging=False)
            except ValueError:
                hlet_msg += _timestamp("SKIPPED: Headerlet not created for %s \n"%fname)
                # update trailer file to log creation of headerlet files
        hlet_msg += _timestamp("Writing Headerlets completed")
        ftrl = open(_trlfile,'a')
        ftrl.write(hlet_msg)
        ftrl.close()

    # If processing was done in a temp working dir, restore results to original
    # processing directory, return to original working dir and remove temp dir
    if newpath:
        _restoreResults(new_processing_dir,orig_processing_dir)
        os.chdir(orig_processing_dir)
        _removeWorkingDir(new_processing_dir)

    # Provide feedback to user
    print _final_msg

def _lowerAsn(asnfile):
    """ Create a copy of the original asn file and change
        the case of all members to lower-case.
    """
    # Start by creating a new name for the ASN table
    _indx = asnfile.find('_asn.fits')
    _new_asn = asnfile[:_indx]+'_pipeline'+asnfile[_indx:]
    if os.path.exists(_new_asn) == True:
        os.remove(_new_asn)
    # copy original ASN table to new table
    shutil.copy(asnfile,_new_asn)

    # Open up the new copy and convert all MEMNAME's to lower-case
    fasn = fits.open(_new_asn,'update')
    for i in xrange(len(fasn[1].data)):
        fasn[1].data[i].setfield('MEMNAME',fasn[1].data[i].field('MEMNAME').lower())
    fasn.close()

    return _new_asn

def _appendTrlFile(trlfile,drizfile):
    """ Append drizfile to already existing trlfile from CALXXX.
    """
    if not os.path.exists(drizfile):
        return
    # Open already existing CALWF3 trailer file for appending
    ftrl = open(trlfile,'a')
    # Open astrodrizzle trailer file
    fdriz = open(drizfile)

    # Read in drizzle comments
    _dlines = fdriz.readlines()

    # Append them to CALWF3 trailer file
    ftrl.writelines(_dlines)

    # Close all files
    ftrl.close()
    fdriz.close()

    # Now, clean up astrodrizzle trailer file
    os.remove(drizfile)


def _timestamp(_process_name):
    """Create formatted time string recognizable by OPUS."""
    _prefix= time.strftime("%Y%j%H%M%S-I-----",time.localtime())
    _lenstr = 60 - len(_process_name)
    return _prefix+_process_name+(_lenstr*'-')+'\n'

def _getTime():
    # Format time values for keywords IRAF-TLM, and DATE
    _ltime = time.localtime(time.time())
    time_str = time.strftime('%H:%M:%S (%d-%b-%Y)',_ltime)

    return time_str

#### Functions used to manage processing in a separate directory/ramdisk
def _createWorkingDir(rootdir,input):
    """
    Create a working directory based on input name under the parent directory specified as rootdir
    """
    # extract rootname from input
    rootname = input[:input.find('_')]
    newdir = os.path.join(rootdir,rootname)
    if not os.path.exists(newdir):
        os.mkdir(newdir)
    return newdir

def _copyToNewWorkingDir(newdir,input):
    """ Copy input file and all related files necessary for processing to the new working directory.

        This function works in a greedy manner, in that all files associated
        with all inputs(have the same rootname) will be copied to the new
        working directory.
    """
    flist = []
    if '_asn.fits' in input:
        asndict = asnutil.readASNTable(input,None)
        flist.append(input[:input.find('_')])
        flist.extend(asndict['order'])
        flist.append(asndict['output'])
    else:
        flist.append(input[:input.find('_')])
    # copy all files related to these rootnames into new dir
    for rootname in flist:
        for fname in glob.glob(rootname+'*'):
            shutil.copy(fname,os.path.join(newdir,fname))

def _restoreResults(newdir,origdir):
    """ Move (not copy) all files from newdir back to the original directory
    """
    for fname in glob.glob(os.path.join(newdir,'*')):
        shutil.move(fname,os.path.join(origdir,os.path.basename(fname)))

def _removeWorkingDir(newdir):
    """ Delete working directory
    """
    os.rmdir(newdir)


#### Functions to support execution from the shell.
def main():

    import getopt

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'bhfin:')
    except getopt.error, e:
        print str(e)
        print __doc__
        print "\t", __version__

    # initialize default values
    help = 0
    force = False
    newdir = None
    inmemory = False
    num_cores = None
    headerlets= True

    # read options
    for opt, value in optlist:
        if opt == "-h":
            help = 1
        if opt == "-f":
            force = True
        if opt == "-i":
            inmemory = True
        if opt == '-n':
            if not value.isdigit():
                print 'ERROR: num_cores value must be an integer!'
                raise ValueError
            num_cores = int(value)
        if opt == '-b':
            # turn off writing headerlets
            headerlets=False
    if len(args) < 1:
        print "syntax: runastrodriz.py [-fhibn] inputFilename [newpath]"
        sys.exit()
    if len(args) > 1:
        newdir = args[-1]
    if (help):
        print __doc__
        print "\t", __version__+'('+__vdate__+')'
    else:
        try:
            process(args[0],force=force,newpath=newdir, inmemory=inmemory,
                    num_cores=num_cores, headerlets=headerlets)
        except Exception, errorobj:
            print str(errorobj)
            print "ERROR: Cannot run astrodrizzle on %s." % sys.argv[1]
            raise Exception, str(errorobj)

    sys.exit()


if __name__ == "__main__":
    main()
